human_data = {}
droid_data = {}


def setup():
    from .schema import Human, Droid
    global human_data, droid_data
    luke = Human(
        id='1000',
        name='Luke Skywalker',
        friends=['1002', '1003', '2000', '2001'],
        appears_in=[4, 5, 6],
        home_planet='Tatooine',
    )

    vader = Human(
        id='1001',
        name='Darth Vader',
        friends=['1004'],
        appears_in=[4, 5, 6],
        home_planet='Tatooine',
    )

    han = Human(
        id='1002',
        name='Han Solo',
        friends=['1000', '1003', '2001'],
        appears_in=[4, 5, 6],
        home_planet=None,
    )

    leia = Human(
        id='1003',
        name='Leia Organa',
        friends=['1000', '1002', '2000', '2001'],
        appears_in=[4, 5, 6],
        home_planet='Alderaan',
    )

    tarkin = Human(
        id='1004',
        name='Wilhuff Tarkin',
        friends=['1001'],
        appears_in=[4],
        home_planet=None,
    )

    human_data = {
        '1000': luke,
        '1001': vader,
        '1002': han,
        '1003': leia,
        '1004': tarkin,
    }

    threepio = Droid(
        id='2000',
        name='C-3PO',
        friends=['1000', '1002', '1003', '2001'],
        appears_in=[4, 5, 6],
        primary_function='Protocol',
    )

    artoo = Droid(
        id='2001',
        name='R2-D2',
        friends=['1000', '1002', '1003'],
        appears_in=[4, 5, 6],
        primary_function='Astromech',
    )

    droid_data = {
        '2000': threepio,
        '2001': artoo,
    }


def get_character(id):
    return human_data.get(id) or droid_data.get(id)


def get_friends(character):
    return map(get_character, character.friends)


def get_hero(episode):
    if episode == 5:
        return human_data['1000']
    return droid_data['2001']


def get_human(id):
    return human_data.get(id)


def get_droid(id):
    return droid_data.get(id)



from graphql.core.type import GraphQLList, GraphQLNonNull, GraphQLString

from graphene.core.schema import Schema

from ..definitions import List, NonNull
from ..scalars import String

schema = Schema()


def test_list_scalar():
    type = schema.T(List(String()))
    assert isinstance(type, GraphQLList)
    assert type.of_type == GraphQLString


def test_nonnull_scalar():
    type = schema.T(NonNull(String()))
    assert isinstance(type, GraphQLNonNull)
    assert type.of_type == GraphQLString


def test_nested_scalars():
    type = schema.T(NonNull(List(String())))
    assert isinstance(type, GraphQLNonNull)
    assert isinstance(type.of_type, GraphQLList)
    assert type.of_type.of_type == GraphQLString



from graphql_relay.node.node import to_global_id

from ..core.types import (Boolean, Field, InputObjectType, Interface, List,
                          Mutation, ObjectType, String)
from ..core.types.argument import ArgumentsGroup
from ..core.types.base import LazyType
from ..core.types.definitions import NonNull
from ..utils import memoize
from .fields import GlobalIDField


class PageInfo(ObjectType):
    has_next_page = Boolean(
        required=True,
        description='When paginating forwards, are there more items?')
    has_previous_page = Boolean(
        required=True,
        description='When paginating backwards, are there more items?')
    start_cursor = String(
        description='When paginating backwards, the cursor to continue.')
    end_cursor = String(
        description='When paginating forwards, the cursor to continue.')


class Edge(ObjectType):
    '''An edge in a connection.'''
    cursor = String(
        required=True, description='A cursor for use in pagination')

    @classmethod
    @memoize
    def for_node(cls, node):
        from graphene.relay.utils import is_node
        assert is_node(node), 'ObjectTypes in a edge have to be Nodes'
        node_field = Field(node, description='The item at the end of the edge')
        return type(
            '%s%s' % (node._meta.type_name, cls._meta.type_name),
            (cls,),
            {'node_type': node, 'node': node_field})


class Connection(ObjectType):
    '''A connection to a list of items.'''
    class Meta:
        type_name = 'DefaultConnection'

    page_info = Field(PageInfo, required=True,
                      description='The Information to aid in pagination')

    _connection_data = None

    @classmethod
    @memoize
    def for_node(cls, node, edge_type=None):
        from graphene.relay.utils import is_node
        edge_type = edge_type or Edge.for_node(node)
        assert is_node(node), 'ObjectTypes in a connection have to be Nodes'
        edges = List(edge_type, description='Information to aid in pagination.')
        return type(
            '%s%s' % (node._meta.type_name, cls._meta.type_name),
            (cls,),
            {'edge_type': edge_type, 'edges': edges})

    def set_connection_data(self, data):
        self._connection_data = data

    def get_connection_data(self):
        return self._connection_data


class BaseNode(object):

    @classmethod
    def _prepare_class(cls):
        from graphene.relay.utils import is_node
        if is_node(cls):
            assert hasattr(
                cls, 'get_node'), 'get_node classmethod not found in %s Node' % cls

    def to_global_id(self):
        type_name = self._meta.type_name
        return to_global_id(type_name, self.id)

    connection_type = Connection
    edge_type = Edge

    @classmethod
    def get_connection_type(cls):
        return cls.connection_type

    @classmethod
    def get_edge_type(cls):
        return cls.edge_type


class Node(BaseNode, Interface):
    '''An object with an ID'''
    id = GlobalIDField()


class MutationInputType(InputObjectType):
    client_mutation_id = String(required=True)


class ClientIDMutation(Mutation):
    client_mutation_id = String(required=True)

    @classmethod
    def _construct_arguments(cls, items):
        assert hasattr(
            cls, 'mutate_and_get_payload'), 'You have to implement mutate_and_get_payload'
        new_input_type = type('{}Input'.format(
            cls._meta.type_name), (MutationInputType, ), items)
        cls.add_to_class('input_type', new_input_type)
        return ArgumentsGroup(input=NonNull(new_input_type))

    @classmethod
    def mutate(cls, instance, args, info):
        input = args.get('input')
        payload = cls.mutate_and_get_payload(input, info)
        client_mutation_id = input.get('client_mutation_id')
        setattr(payload, 'client_mutation_id', client_mutation_id)
        return payload



import graphene
from graphene import relay, resolve_only_args

from .data import create_ship, get_empire, get_faction, get_rebels, get_ship

schema = graphene.Schema(name='Starwars Relay Schema')


class Ship(relay.Node):
    '''A ship in the Star Wars saga'''
    name = graphene.String(description='The name of the ship.')

    @classmethod
    def get_node(cls, id):
        return get_ship(id)


class Faction(relay.Node):
    '''A faction in the Star Wars saga'''
    name = graphene.String(description='The name of the faction.')
    ships = relay.ConnectionField(
        Ship, description='The ships used by the faction.')

    @resolve_only_args
    def resolve_ships(self, **args):
        # Transform the instance ship_ids into real instances
        return [get_ship(ship_id) for ship_id in self.ships]

    @classmethod
    def get_node(cls, id):
        return get_faction(id)


class IntroduceShip(relay.ClientIDMutation):

    class Input:
        ship_name = graphene.String(required=True)
        faction_id = graphene.String(required=True)

    ship = graphene.Field(Ship)
    faction = graphene.Field(Faction)

    @classmethod
    def mutate_and_get_payload(cls, input, info):
        ship_name = input.get('ship_name')
        faction_id = input.get('faction_id')
        ship = create_ship(ship_name, faction_id)
        faction = get_faction(faction_id)
        return IntroduceShip(ship=ship, faction=faction)


class Query(graphene.ObjectType):
    rebels = graphene.Field(Faction)
    empire = graphene.Field(Faction)
    node = relay.NodeField()

    @resolve_only_args
    def resolve_rebels(self):
        return get_rebels()

    @resolve_only_args
    def resolve_empire(self):
        return get_empire()


class Mutation(graphene.ObjectType):
    introduce_ship = graphene.Field(IntroduceShip)


schema.query = Query
schema.mutation = Mutation



from graphql.core.execution.middlewares.utils import (resolver_has_tag,
                                                      tag_resolver)
from graphql.core.type import (GraphQLInterfaceType, GraphQLObjectType,
                               GraphQLUnionType)
from py.test import raises

from graphene.core.schema import Schema
from graphene.core.types import Int, Interface, String


class Character(Interface):
    '''Character description'''
    name = String()

    class Meta:
        type_name = 'core_Character'


class Human(Character):
    '''Human description'''
    friends = String()

    class Meta:
        type_name = 'core_Human'

    @property
    def readonly_prop(self):
        return 'readonly'

    @property
    def write_prop(self):
        return self._write_prop

    @write_prop.setter
    def write_prop(self, value):
        self._write_prop = value


class Droid(Character):
    '''Droid description'''


class CharacterType(Droid, Human):
    '''Union Type'''

schema = Schema()


def test_interface():
    object_type = schema.T(Character)
    assert Character._meta.is_interface is True
    assert isinstance(object_type, GraphQLInterfaceType)
    assert Character._meta.type_name == 'core_Character'
    assert object_type.description == 'Character description'
    assert list(object_type.get_fields().keys()) == ['name']


def test_interface_cannot_initialize():
    with raises(Exception) as excinfo:
        Character()
    assert 'An interface cannot be initialized' == str(excinfo.value)


def test_union():
    object_type = schema.T(CharacterType)
    assert CharacterType._meta.is_union is True
    assert isinstance(object_type, GraphQLUnionType)
    assert object_type.description == 'Union Type'


def test_union_cannot_initialize():
    with raises(Exception) as excinfo:
        CharacterType()
    assert 'An union cannot be initialized' == str(excinfo.value)


def test_interface_resolve_type():
    resolve_type = Character.resolve_type(schema, Human(object()))
    assert isinstance(resolve_type, GraphQLObjectType)


def test_object_type():
    object_type = schema.T(Human)
    assert Human._meta.is_interface is False
    assert Human._meta.type_name == 'core_Human'
    assert isinstance(object_type, GraphQLObjectType)
    assert object_type.description == 'Human description'
    assert list(object_type.get_fields().keys()) == ['name', 'friends']
    assert object_type.get_interfaces() == [schema.T(Character)]
    assert Human._meta.fields_map['name'].object_type == Human


def test_object_type_container():
    h = Human(name='My name')
    assert h.name == 'My name'


def test_object_type_set_properties():
    h = Human(readonly_prop='custom', write_prop='custom')
    assert h.readonly_prop == 'readonly'
    assert h.write_prop == 'custom'


def test_object_type_container_invalid_kwarg():
    with raises(TypeError):
        Human(invalid='My name')


def test_object_type_container_too_many_args():
    with raises(IndexError):
        Human('Peter', 'No friends :(', None)


def test_field_clashes():
    with raises(Exception) as excinfo:
        class Droid(Character):
            name = Int()

    assert 'clashes' in str(excinfo.value)


def test_fields_inherited_should_be_different():
    assert Character._meta.fields_map['name'] != Human._meta.fields_map['name']


def test_field_mantain_resolver_tags():
    class Droid(Character):
        name = String()

        def resolve_name(self, *args):
            return 'My Droid'

        tag_resolver(resolve_name, 'test')

    field = schema.T(Droid._meta.fields_map['name'])
    assert resolver_has_tag(field.resolver, 'test')


def test_type_has_nonnull():
    class Droid(Character):
        name = String()

    assert Droid.NonNull.of_type == Droid


def test_type_has_list():
    class Droid(Character):
        name = String()

    assert Droid.List.of_type == Droid



from py.test import raises

from ..proxy_snake_dict import ProxySnakeDict


def test_proxy_snake_dict():
    my_data = {'one': 1, 'two': 2, 'none': None,
               'threeOrFor': 3, 'inside': {'otherCamelCase': 3}}
    p = ProxySnakeDict(my_data)
    assert 'one' in p
    assert 'two' in p
    assert 'threeOrFor' in p
    assert 'none' in p
    assert len(p) == len(my_data)
    assert p['none'] is None
    assert p.get('none') is None
    assert p.get('none_existent') is None
    assert 'three_or_for' in p
    assert p.get('three_or_for') == 3
    assert 'inside' in p
    assert 'other_camel_case' in p['inside']
    assert sorted(
        p.items()) == sorted(
        list(
            [('inside', ProxySnakeDict({'other_camel_case': 3})),
             ('none', None),
             ('three_or_for', 3),
             ('two', 2),
             ('one', 1)]))


def test_proxy_snake_dict_as_kwargs():
    my_data = {'myData': 1}
    p = ProxySnakeDict(my_data)

    def func(**kwargs):
        return kwargs.get('my_data')
    assert func(**p) == 1


def test_proxy_snake_dict_repr():
    my_data = {'myData': 1}
    p = ProxySnakeDict(my_data)

    assert repr(p) == "<ProxySnakeDict {'my_data': 1}>"


def test_proxy_snake_dict_set():
    p = ProxySnakeDict({})
    with raises(TypeError):
        p['a'] = 2


def test_proxy_snake_dict_delete():
    p = ProxySnakeDict({})
    with raises(TypeError):
        del p['a']



import collections

from graphql.core.type import GraphQLEnumType

from ..misc import enum_to_graphql_enum

item = collections.namedtuple('type', 'name value')


class MyCustomEnum(list):
    __name__ = 'MyName'


def test_enum_to_graphql_enum():
    assert isinstance(enum_to_graphql_enum(
        MyCustomEnum([item('k', 'v')])), GraphQLEnumType)



from graphql.core.type import (GraphQLBoolean, GraphQLFloat, GraphQLID,
                               GraphQLInt, GraphQLScalarType, GraphQLString)

from .base import MountedType


class String(MountedType):
    T = GraphQLString


class Int(MountedType):
    T = GraphQLInt


class Boolean(MountedType):
    T = GraphQLBoolean


class ID(MountedType):
    T = GraphQLID


class Float(MountedType):
    T = GraphQLFloat


class Scalar(MountedType):

    @classmethod
    def internal_type(cls, schema):
        serialize = getattr(cls, 'serialize')
        parse_literal = getattr(cls, 'parse_literal')
        parse_value = getattr(cls, 'parse_value')

        return GraphQLScalarType(
            name=cls.__name__,
            description=cls.__doc__,
            serialize=serialize,
            parse_value=parse_value,
            parse_literal=parse_literal
        )



import inspect

from django.db import models

from ...core.options import Options
from ...relay.types import Node
from ...relay.utils import is_node

VALID_ATTRS = ('model', 'only_fields', 'exclude_fields')


def is_base(cls):
    from graphene.contrib.django.types import DjangoObjectType
    return DjangoObjectType in cls.__bases__


class DjangoOptions(Options):

    def __init__(self, *args, **kwargs):
        self.model = None
        super(DjangoOptions, self).__init__(*args, **kwargs)
        self.valid_attrs += VALID_ATTRS
        self.only_fields = None
        self.exclude_fields = []

    def contribute_to_class(self, cls, name):
        super(DjangoOptions, self).contribute_to_class(cls, name)
        if is_node(cls):
            self.exclude_fields = list(self.exclude_fields) + ['id']
            self.interfaces.append(Node)
        if not is_node(cls) and not is_base(cls):
            return
        if not self.model:
            raise Exception(
                'Django ObjectType %s must have a model in the Meta class attr' %
                cls)
        elif not inspect.isclass(self.model) or not issubclass(self.model, models.Model):
            raise Exception('Provided model in %s is not a Django model' % cls)



from graphql.core.type import (GraphQLField, GraphQLInputObjectField,
                               GraphQLString)

from graphene.core.schema import Schema
from graphene.core.types import InputObjectType, ObjectType

from ..base import LazyType
from ..definitions import List
from ..field import Field, InputField
from ..scalars import String


def test_field_internal_type():
    resolver = lambda *args: 'RESOLVED'

    field = Field(String, description='My argument', resolver=resolver)

    class Query(ObjectType):
        my_field = field
    schema = Schema(query=Query)

    type = schema.T(field)
    assert field.name == 'myField'
    assert field.attname == 'my_field'
    assert isinstance(type, GraphQLField)
    assert type.description == 'My argument'
    assert type.resolver(None, {}, None) == 'RESOLVED'
    assert type.type == GraphQLString


def test_field_objectype_resolver():
    field = Field(String)

    class Query(ObjectType):
        my_field = field

        def resolve_my_field(self, *args, **kwargs):
            '''Custom description'''
            return 'RESOLVED'

    schema = Schema(query=Query)

    type = schema.T(field)
    assert isinstance(type, GraphQLField)
    assert type.description == 'Custom description'
    assert type.resolver(Query(), {}, None) == 'RESOLVED'


def test_field_custom_name():
    field = Field(None, name='my_customName')

    class MyObjectType(ObjectType):
        my_field = field

    assert field.name == 'my_customName'
    assert field.attname == 'my_field'


def test_field_self():
    field = Field('self', name='my_customName')

    class MyObjectType(ObjectType):
        my_field = field

    schema = Schema()

    assert schema.T(field).type == schema.T(MyObjectType)


def test_field_eq():
    field = Field('self', name='my_customName')
    field2 = Field('self', name='my_customName')
    assert field == field
    assert field2 != field


def test_field_mounted():
    field = Field(List('MyObjectType'), name='my_customName')

    class MyObjectType(ObjectType):
        my_field = field

    assert field.parent == MyObjectType
    assert field.type.parent == MyObjectType


def test_field_string_reference():
    field = Field('MyObjectType', name='my_customName')

    class MyObjectType(ObjectType):
        my_field = field

    schema = Schema(query=MyObjectType)

    assert isinstance(field.type, LazyType)
    assert schema.T(field.type) == schema.T(MyObjectType)


def test_field_custom_arguments():
    field = Field(None, name='my_customName', p=String())

    args = field.arguments
    assert 'p' in args


def test_inputfield_internal_type():
    field = InputField(String, description='My input field', default='3')

    class MyObjectType(InputObjectType):
        my_field = field

    class Query(ObjectType):
        input_ot = Field(MyObjectType)

    schema = Schema(query=MyObjectType)

    type = schema.T(field)
    assert field.name == 'myField'
    assert field.attname == 'my_field'
    assert isinstance(type, GraphQLInputObjectField)
    assert type.description == 'My input field'
    assert type.default_value == '3'



from mock import patch

from graphene.core.types import InputObjectType, ObjectType

from ..argument import Argument
from ..base import MountedType, OrderedType
from ..definitions import List, NonNull
from ..field import Field, InputField


def test_orderedtype_equal():
    a = OrderedType()
    assert a == a
    assert hash(a) == hash(a)


def test_orderedtype_different():
    a = OrderedType()
    b = OrderedType()
    assert a != b
    assert hash(a) != hash(b)
    assert a < b
    assert b > a


@patch('graphene.core.types.field.Field')
def test_type_as_field_called(Field):
    resolver = lambda x: x
    a = MountedType(2, description='A', resolver=resolver)
    a.as_field()
    Field.assert_called_with(
        a,
        2,
        _creation_counter=a.creation_counter,
        description='A',
        resolver=resolver)


@patch('graphene.core.types.argument.Argument')
def test_type_as_argument_called(Argument):
    a = MountedType(2, description='A')
    a.as_argument()
    Argument.assert_called_with(
        a, 2, _creation_counter=a.creation_counter, description='A')


def test_type_as_field():
    resolver = lambda x: x

    class MyObjectType(ObjectType):
        t = MountedType(description='A', resolver=resolver)

    fields_map = MyObjectType._meta.fields_map
    field = fields_map.get('t')
    assert isinstance(field, Field)
    assert field.description == 'A'
    assert field.object_type == MyObjectType


def test_type_as_inputfield():
    class MyObjectType(InputObjectType):
        t = MountedType(description='A')

    fields_map = MyObjectType._meta.fields_map
    field = fields_map.get('t')
    assert isinstance(field, InputField)
    assert field.description == 'A'
    assert field.object_type == MyObjectType


def test_type_as_argument():
    a = MountedType(description='A')
    argument = a.as_argument()
    assert isinstance(argument, Argument)


def test_type_as_list():
    m = MountedType(2, 3, my_c='A')
    a = m.List

    assert isinstance(a, List)
    assert a.of_type == m
    assert a.args == (2, 3)
    assert a.kwargs == {'my_c': 'A'}


def test_type_as_nonnull():
    m = MountedType(2, 3, my_c='A')
    a = m.NonNull

    assert isinstance(a, NonNull)
    assert a.of_type == m
    assert a.args == (2, 3)
    assert a.kwargs == {'my_c': 'A'}



from graphql_django_view import GraphQLView as BaseGraphQLView


class GraphQLView(BaseGraphQLView):
    graphene_schema = None

    def __init__(self, schema, **kwargs):
        super(GraphQLView, self).__init__(
            graphene_schema=schema,
            schema=schema.schema,
            executor=schema.executor,
            **kwargs
        )

    def get_root_value(self, request):
        return self.graphene_schema.query(super(GraphQLView, self).get_root_value(request))



import json


def format_response(response):
    return json.loads(response.content.decode())


def test_client_get_good_query(settings, client):
    settings.ROOT_URLCONF = 'graphene.contrib.django.tests.test_urls'
    response = client.get('/graphql', {'query': '{ headline }'})
    json_response = format_response(response)
    expected_json = {
        'data': {
            'headline': None
        }
    }
    assert json_response == expected_json


def test_client_get_good_query_with_raise(settings, client):
    settings.ROOT_URLCONF = 'graphene.contrib.django.tests.test_urls'
    response = client.get('/graphql', {'query': '{ raises }'})
    json_response = format_response(response)
    assert json_response['errors'][0]['message'] == 'This field should raise exception'
    assert json_response['data']['raises'] is None


def test_client_post_good_query_json(settings, client):
    settings.ROOT_URLCONF = 'graphene.contrib.django.tests.test_urls'
    response = client.post(
        '/graphql', json.dumps({'query': '{ headline }'}), 'application/json')
    json_response = format_response(response)
    expected_json = {
        'data': {
            'headline': None
        }
    }
    assert json_response == expected_json


def test_client_post_good_query_graphql(settings, client):
    settings.ROOT_URLCONF = 'graphene.contrib.django.tests.test_urls'
    response = client.post(
        '/graphql', '{ headline }', 'application/graphql')
    json_response = format_response(response)
    expected_json = {
        'data': {
            'headline': None
        }
    }
    assert json_response == expected_json



from django.db import models
from django.db.models.manager import Manager
from django.db.models.query import QuerySet

from ...utils import LazyMap


def get_type_for_model(schema, model):
    schema = schema
    types = schema.types.values()
    for _type in types:
        type_model = hasattr(_type, '_meta') and getattr(
            _type._meta, 'model', None)
        if model == type_model:
            return _type


def get_reverse_fields(model):
    for name, attr in model.__dict__.items():
        related = getattr(attr, 'related', None)
        if isinstance(related, models.ManyToOneRel):
            yield related


def lazy_map(value, func):
    if isinstance(value, Manager):
        value = value.get_queryset()
    if isinstance(value, QuerySet):
        return LazyMap(value, func)
    return value



from collections import OrderedDict

from graphql.core.type import GraphQLEnumType, GraphQLEnumValue


def enum_to_graphql_enum(enumeration):
    return GraphQLEnumType(
        name=enumeration.__name__,
        values=OrderedDict([(it.name, GraphQLEnumValue(it.value))
                            for it in enumeration]),
        description=enumeration.__doc__
    )



import copy
import inspect
from collections import OrderedDict
from functools import partial

import six
from graphql.core.type import (GraphQLInputObjectType, GraphQLInterfaceType,
                               GraphQLObjectType, GraphQLUnionType)

from graphene import signals

from ..exceptions import SkipField
from ..options import Options
from .argument import ArgumentsGroup
from .base import BaseType
from .definitions import List, NonNull


class ObjectTypeMeta(type):
    options_cls = Options

    def is_interface(cls, parents):
        return Interface in parents

    def is_mutation(cls, parents):
        return issubclass(cls, Mutation)

    def __new__(cls, name, bases, attrs):
        super_new = super(ObjectTypeMeta, cls).__new__
        parents = [b for b in bases if isinstance(b, cls)]
        if not parents:
            # If this isn't a subclass of Model, don't do anything special.
            return super_new(cls, name, bases, attrs)

        module = attrs.pop('__module__', None)
        doc = attrs.pop('__doc__', None)
        new_class = super_new(cls, name, bases, {
            '__module__': module,
            '__doc__': doc
        })
        attr_meta = attrs.pop('Meta', None)
        if not attr_meta:
            meta = None
            # meta = getattr(new_class, 'Meta', None)
        else:
            meta = attr_meta

        getattr(new_class, '_meta', None)

        new_class.add_to_class('_meta', new_class.options_cls(meta))

        new_class._meta.is_interface = new_class.is_interface(parents)
        new_class._meta.is_mutation = new_class.is_mutation(parents)
        union_types = [p for p in parents if issubclass(p, BaseObjectType)]

        new_class._meta.is_union = len(union_types) > 1
        new_class._meta.types = union_types

        assert not (
            new_class._meta.is_interface and new_class._meta.is_mutation)

        assert not (
            new_class._meta.is_interface and new_class._meta.is_union)

        # Add all attributes to the class.
        for obj_name, obj in attrs.items():
            new_class.add_to_class(obj_name, obj)

        if new_class._meta.is_mutation:
            assert hasattr(
                new_class, 'mutate'), "All mutations must implement mutate method"

        new_class.add_extra_fields()

        new_fields = new_class._meta.local_fields
        assert not(new_class._meta.is_union and new_fields), 'An union cannot have extra fields'

        field_names = {f.name: f for f in new_fields}

        for base in parents:
            if not hasattr(base, '_meta'):
                # Things without _meta aren't functional models, so they're
                # uninteresting parents.
                continue
            # if base._meta.schema != new_class._meta.schema:
            #     raise Exception('The parent schema is not the same')

            parent_fields = base._meta.local_fields
            # Check for clashes between locally declared fields and those
            # on the base classes (we cannot handle shadowed fields at the
            # moment).
            for field in parent_fields:
                if field.name in field_names and field.type.__class__ != field_names[
                        field.name].type.__class__:
                    raise Exception(
                        'Local field %r in class %r (%r) clashes '
                        'with field with similar name from '
                        'Interface %s (%r)' % (
                            field.name,
                            new_class.__name__,
                            field.__class__,
                            base.__name__,
                            field_names[field.name].__class__)
                    )
                new_field = copy.copy(field)
                new_class.add_to_class(field.attname, new_field)

            new_class._meta.parents.append(base)
            if base._meta.is_interface:
                new_class._meta.interfaces.append(base)
            # new_class._meta.parents.extend(base._meta.parents)

        setattr(new_class, 'NonNull', NonNull(new_class))
        setattr(new_class, 'List', List(new_class))

        new_class._prepare()
        return new_class

    def add_extra_fields(cls):
        pass

    def _prepare(cls):
        if hasattr(cls, '_prepare_class'):
            cls._prepare_class()
        signals.class_prepared.send(cls)

    def add_to_class(cls, name, value):
        # We should call the contribute_to_class method only if it's bound
        if not inspect.isclass(value) and hasattr(
                value, 'contribute_to_class'):
            value.contribute_to_class(cls, name)
        else:
            setattr(cls, name, value)


class BaseObjectType(BaseType):

    def __new__(cls, *args, **kwargs):
        if cls._meta.is_interface:
            raise Exception("An interface cannot be initialized")
        if cls._meta.is_union:
            raise Exception("An union cannot be initialized")
        return super(BaseObjectType, cls).__new__(cls)

    def __init__(self, *args, **kwargs):
        signals.pre_init.send(self.__class__, args=args, kwargs=kwargs)
        args_len = len(args)
        fields = self._meta.fields
        if args_len > len(fields):
            # Daft, but matches old exception sans the err msg.
            raise IndexError("Number of args exceeds number of fields")
        fields_iter = iter(fields)

        if not kwargs:
            for val, field in zip(args, fields_iter):
                setattr(self, field.attname, val)
        else:
            for val, field in zip(args, fields_iter):
                setattr(self, field.attname, val)
                kwargs.pop(field.attname, None)

        for field in fields_iter:
            try:
                val = kwargs.pop(field.attname)
                setattr(self, field.attname, val)
            except KeyError:
                pass

        if kwargs:
            for prop in list(kwargs):
                try:
                    if isinstance(getattr(self.__class__, prop), property):
                        setattr(self, prop, kwargs.pop(prop))
                except AttributeError:
                    pass
            if kwargs:
                raise TypeError(
                    "'%s' is an invalid keyword argument for this function" %
                    list(kwargs)[0])

        signals.post_init.send(self.__class__, instance=self)

    @classmethod
    def resolve_type(cls, schema, instance, *args):
        return schema.T(instance.__class__)

    @classmethod
    def internal_type(cls, schema):
        if cls._meta.is_interface:
            return GraphQLInterfaceType(
                cls._meta.type_name,
                description=cls._meta.description,
                resolve_type=partial(cls.resolve_type, schema),
                fields=partial(cls.get_fields, schema)
            )
        elif cls._meta.is_union:
            return GraphQLUnionType(
                cls._meta.type_name,
                types=cls._meta.types,
                description=cls._meta.description,
            )
        return GraphQLObjectType(
            cls._meta.type_name,
            description=cls._meta.description,
            interfaces=[schema.T(i) for i in cls._meta.interfaces],
            fields=partial(cls.get_fields, schema),
            is_type_of=getattr(cls, 'is_type_of', None)
        )

    @classmethod
    def get_fields(cls, schema):
        fields = []
        for field in cls._meta.fields:
            try:
                fields.append((field.name, schema.T(field)))
            except SkipField:
                continue

        return OrderedDict(fields)


class Interface(six.with_metaclass(ObjectTypeMeta, BaseObjectType)):
    pass


class ObjectType(six.with_metaclass(ObjectTypeMeta, BaseObjectType)):
    pass


class Mutation(six.with_metaclass(ObjectTypeMeta, BaseObjectType)):

    @classmethod
    def _construct_arguments(cls, items):
        return ArgumentsGroup(**items)

    @classmethod
    def _prepare_class(cls):
        input_class = getattr(cls, 'Input', None)
        if input_class:
            items = dict(vars(input_class))
            items.pop('__dict__', None)
            items.pop('__doc__', None)
            items.pop('__module__', None)
            items.pop('__weakref__', None)
            cls.add_to_class('arguments', cls._construct_arguments(items))
            delattr(cls, 'Input')

    @classmethod
    def get_arguments(cls):
        return cls.arguments


class InputObjectType(ObjectType):

    @classmethod
    def internal_type(cls, schema):
        return GraphQLInputObjectType(
            cls._meta.type_name,
            description=cls._meta.description,
            fields=partial(cls.get_fields, schema),
        )



data = {}


def setup():
    global data

    from .schema import Ship, Faction
    xwing = Ship(
        id='1',
        name='X-Wing',
    )

    ywing = Ship(
        id='2',
        name='Y-Wing',
    )

    awing = Ship(
        id='3',
        name='A-Wing',
    )

    # Yeah, technically it's Corellian. But it flew in the service of the rebels,
    # so for the purposes of this demo it's a rebel ship.
    falcon = Ship(
        id='4',
        name='Millenium Falcon',
    )

    homeOne = Ship(
        id='5',
        name='Home One',
    )

    tieFighter = Ship(
        id='6',
        name='TIE Fighter',
    )

    tieInterceptor = Ship(
        id='7',
        name='TIE Interceptor',
    )

    executor = Ship(
        id='8',
        name='Executor',
    )

    rebels = Faction(
        id='1',
        name='Alliance to Restore the Republic',
        ships=['1', '2', '3', '4', '5']
    )

    empire = Faction(
        id='2',
        name='Galactic Empire',
        ships=['6', '7', '8']
    )

    data = {
        'Faction': {
            '1': rebels,
            '2': empire
        },
        'Ship': {
            '1': xwing,
            '2': ywing,
            '3': awing,
            '4': falcon,
            '5': homeOne,
            '6': tieFighter,
            '7': tieInterceptor,
            '8': executor
        }
    }


def create_ship(ship_name, faction_id):
    from .schema import Ship
    next_ship = len(data['Ship'].keys()) + 1
    new_ship = Ship(
        id=str(next_ship),
        name=ship_name
    )
    data['Ship'][new_ship.id] = new_ship
    data['Faction'][faction_id].ships.append(new_ship.id)
    return new_ship


def get_ship(_id):
    return data['Ship'][_id]


def get_faction(_id):
    return data['Faction'][_id]


def get_rebels():
    return get_faction('1')


def get_empire():
    return get_faction('2')



from ...core.exceptions import SkipField
from ...core.fields import Field
from ...core.types.base import FieldType
from ...core.types.definitions import List
from ...relay import ConnectionField
from ...relay.utils import is_node
from .utils import get_type_for_model, lazy_map


class DjangoConnectionField(ConnectionField):

    def wrap_resolved(self, value, instance, args, info):
        schema = info.schema.graphene_schema
        return lazy_map(value, self.type)


class LazyListField(Field):

    def get_type(self, schema):
        return List(self.type)

    def resolver(self, instance, args, info):
        schema = info.schema.graphene_schema
        resolved = super(LazyListField, self).resolver(instance, args, info)
        return lazy_map(resolved, self.type)


class ConnectionOrListField(Field):

    def internal_type(self, schema):
        model_field = self.type
        field_object_type = model_field.get_object_type(schema)
        if not field_object_type:
            raise SkipField()
        if is_node(field_object_type):
            field = DjangoConnectionField(field_object_type)
        else:
            field = LazyListField(field_object_type)
        field.contribute_to_class(self.object_type, self.name)
        return schema.T(field)


class DjangoModelField(FieldType):

    def __init__(self, model, *args, **kwargs):
        self.model = model
        super(DjangoModelField, self).__init__(*args, **kwargs)

    def internal_type(self, schema):
        _type = self.get_object_type(schema)
        if not _type and self.parent._meta.only_fields:
            raise Exception(
                "Model %r is not accessible by the schema. "
                "You can either register the type manually "
                "using @schema.register. "
                "Or disable the field in %s" % (
                    self.model,
                    self.parent,
                )
            )
        if not _type:
            raise SkipField()
        return schema.T(_type)

    def get_object_type(self, schema):
        return get_type_for_model(schema, self.model)



from __future__ import absolute_import

from django.db import models


class Pet(models.Model):
    name = models.CharField(max_length=30)


class Reporter(models.Model):
    first_name = models.CharField(max_length=30)
    last_name = models.CharField(max_length=30)
    email = models.EmailField()
    pets = models.ManyToManyField('self')

    def __str__(self):              # __unicode__ on Python 2
        return "%s %s" % (self.first_name, self.last_name)

    class Meta:
        app_label = 'contrib_django'


class Article(models.Model):
    headline = models.CharField(max_length=100)
    pub_date = models.DateField()
    reporter = models.ForeignKey(Reporter, related_name='articles')

    def __str__(self):              # __unicode__ on Python 2
        return self.headline

    class Meta:
        ordering = ('headline',)
        app_label = 'contrib_django'



from .str_converters import to_camel_case, to_snake_case
from .proxy_snake_dict import ProxySnakeDict
from .caching import cached_property, memoize
from .lazymap import LazyMap
from .misc import enum_to_graphql_enum
from .resolve_only_args import resolve_only_args


__all__ = ['to_camel_case', 'to_snake_case', 'ProxySnakeDict',
           'cached_property', 'memoize', 'LazyMap', 'enum_to_graphql_enum',
           'resolve_only_args']



from collections import OrderedDict
from functools import wraps
from itertools import chain

from graphql.core.type import GraphQLArgument

from ...utils import ProxySnakeDict, to_camel_case
from .base import ArgumentType, BaseType, OrderedType


class Argument(OrderedType):

    def __init__(self, type, description=None, default=None,
                 name=None, _creation_counter=None):
        super(Argument, self).__init__(_creation_counter=_creation_counter)
        self.name = name
        self.type = type
        self.description = description
        self.default = default

    def internal_type(self, schema):
        return GraphQLArgument(
            schema.T(self.type),
            self.default, self.description)

    def __repr__(self):
        return self.name


class ArgumentsGroup(BaseType):

    def __init__(self, *args, **kwargs):
        arguments = to_arguments(*args, **kwargs)
        self.arguments = OrderedDict([(arg.name, arg) for arg in arguments])

    def internal_type(self, schema):
        return OrderedDict([(arg.name, schema.T(arg))
                            for arg in self.arguments.values()])

    def __len__(self):
        return len(self.arguments)

    def __iter__(self):
        return iter(self.arguments)

    def __contains__(self, *args):
        return self.arguments.__contains__(*args)

    def __getitem__(self, *args):
        return self.arguments.__getitem__(*args)


def to_arguments(*args, **kwargs):
    arguments = {}
    iter_arguments = chain(kwargs.items(), [(None, a) for a in args])

    for name, arg in iter_arguments:
        if isinstance(arg, Argument):
            argument = arg
        elif isinstance(arg, ArgumentType):
            argument = arg.as_argument()
        else:
            raise ValueError('Unknown argument %s=%r' % (name, arg))

        if name:
            argument.name = to_camel_case(name)
        assert argument.name, 'Argument in field must have a name'
        assert argument.name not in arguments, 'Found more than one Argument with same name {}'.format(
            argument.name)
        arguments[argument.name] = argument

    return sorted(arguments.values())


def snake_case_args(resolver):
    @wraps(resolver)
    def wrapped_resolver(instance, args, info):
        return resolver(instance, ProxySnakeDict(args), info)

    return wrapped_resolver



import pytest

from ..data import initialize
from ..schema import schema

pytestmark = pytest.mark.django_db


def test_mutations():
    initialize()

    query = '''
    mutation MyMutation {
      introduceShip(input:{clientMutationId:"abc", shipName: "Peter", factionId: "1"}) {
        ship {
          id
          name
        }
        faction {
          name
          ships {
            edges {
              node {
                id
                name
              }
            }
          }
        }
      }
    }
    '''
    expected = {
        'introduceShip': {
            'ship': {
                'id': 'U2hpcDo5',
                'name': 'Peter'
            },
            'faction': {
                'name': 'Alliance to Restore the Republic',
                'ships': {
                    'edges': [{
                        'node': {
                            'id': 'U2hpcDox',
                            'name': 'X-Wing'
                        }
                    }, {
                        'node': {
                            'id': 'U2hpcDoy',
                            'name': 'Y-Wing'
                        }
                    }, {
                        'node': {
                            'id': 'U2hpcDoz',
                            'name': 'A-Wing'
                        }
                    }, {
                        'node': {
                            'id': 'U2hpcDo0',
                            'name': 'Millenium Falcon'
                        }
                    }, {
                        'node': {
                            'id': 'U2hpcDo1',
                            'name': 'Home One'
                        }
                    }, {
                        'node': {
                            'id': 'U2hpcDo5',
                            'name': 'Peter'
                        }
                    }]
                },
            }
        }
    }
    result = schema.execute(query)
    assert not result.errors
    assert result.data == expected



from graphql.core.type import (GraphQLBoolean, GraphQLFloat, GraphQLID,
                               GraphQLInt, GraphQLScalarType, GraphQLString)

from graphene.core.schema import Schema

from ..scalars import ID, Boolean, Float, Int, Scalar, String

schema = Schema()


def test_string_scalar():
    assert schema.T(String()) == GraphQLString


def test_int_scalar():
    assert schema.T(Int()) == GraphQLInt


def test_boolean_scalar():
    assert schema.T(Boolean()) == GraphQLBoolean


def test_id_scalar():
    assert schema.T(ID()) == GraphQLID


def test_float_scalar():
    assert schema.T(Float()) == GraphQLFloat


def test_custom_scalar():
    import datetime
    from graphql.core.language import ast

    class DateTimeScalar(Scalar):
        '''DateTimeScalar Documentation'''
        @staticmethod
        def serialize(dt):
            return dt.isoformat()

        @staticmethod
        def parse_literal(node):
            if isinstance(node, ast.StringValue):
                return datetime.datetime.strptime(
                    node.value, "%Y-%m-%dT%H:%M:%S.%f")

        @staticmethod
        def parse_value(value):
            return datetime.datetime.strptime(value, "%Y-%m-%dT%H:%M:%S.%f")

    scalar_type = schema.T(DateTimeScalar)
    assert isinstance(scalar_type, GraphQLScalarType)
    assert scalar_type.name == 'DateTimeScalar'
    assert scalar_type.description == 'DateTimeScalar Documentation'



class LazyMap(object):

    def __init__(self, origin, _map, state=None):
        self._origin = origin
        self._origin_iter = origin.__iter__()
        self._state = state or []
        self._finished = False
        self._map = _map

    def __iter__(self):
        return self if not self._finished else iter(self._state)

    def iter(self):
        return self.__iter__()

    def __len__(self):
        return self._origin.__len__()

    def __next__(self):
        try:
            n = next(self._origin_iter)
            n = self._map(n)
        except StopIteration as e:
            self._finished = True
            raise e
        else:
            self._state.append(n)
            return n

    def next(self):
        return self.__next__()

    def __getitem__(self, key):
        item = self._origin[key]
        if isinstance(key, slice):
            return LazyMap(item, self._map)
        return self._map(item)

    def __getattr__(self, name):
        return getattr(self._origin, name)

    def __repr__(self):
        return "<LazyMap %s>" % repr(self._origin)



from graphql.core.type import GraphQLID, GraphQLNonNull

import graphene
from graphene import relay

schema = graphene.Schema()


class MyConnection(relay.Connection):
    my_custom_field = graphene.String(
        resolver=lambda instance, *_: 'Custom')


class MyNode(relay.Node):
    name = graphene.String()

    @classmethod
    def get_node(cls, id):
        return MyNode(id=id, name='mo')


class Query(graphene.ObjectType):
    my_node = relay.NodeField(MyNode)
    all_my_nodes = relay.ConnectionField(
        MyNode, connection_type=MyConnection, customArg=graphene.String())

    def resolve_all_my_nodes(self, args, info):
        custom_arg = args.get('customArg')
        assert custom_arg == "1"
        return [MyNode(name='my')]

schema.query = Query


def test_nodefield_query():
    query = '''
    query RebelsShipsQuery {
      myNode(id:"TXlOb2RlOjE=") {
        id
        name
      },
      false: myNode(id:"WrongNodeId") {
        id
        name
      },
      allMyNodes (customArg:"1") {
        edges {
          node {
            name
          }
        },
        myCustomField
        pageInfo {
          hasNextPage
        }
      }
    }
    '''
    expected = {
        'myNode': {
            'id': 'TXlOb2RlOjE=',
            'name': 'mo'
        },
        'false': None,
        'allMyNodes': {
            'edges': [{
                'node': {
                    'name': 'my'
                }
            }],
            'myCustomField': 'Custom',
            'pageInfo': {
                'hasNextPage': False,
            }
        }
    }
    result = schema.execute(query)
    assert not result.errors
    assert result.data == expected


def test_nodeidfield():
    id_field = MyNode._meta.fields_map['id']
    id_field_type = schema.T(id_field)
    assert isinstance(id_field_type.type, GraphQLNonNull)
    assert id_field_type.type.of_type == GraphQLID



from django.db import models
from py.test import raises

import graphene
from graphene.contrib.django.converter import convert_django_field
from graphene.contrib.django.fields import (ConnectionOrListField,
                                            DjangoModelField)

from .models import Article, Reporter


def assert_conversion(django_field, graphene_field, *args):
    field = django_field(*args, help_text='Custom Help Text')
    graphene_type = convert_django_field(field)
    assert isinstance(graphene_type, graphene_field)
    field = graphene_type.as_field()
    assert field.description == 'Custom Help Text'
    return field


def test_should_unknown_django_field_raise_exception():
    with raises(Exception) as excinfo:
        convert_django_field(None)
    assert 'Don\'t know how to convert the Django field' in str(excinfo.value)


def test_should_date_convert_string():
    assert_conversion(models.DateField, graphene.String)


def test_should_char_convert_string():
    assert_conversion(models.CharField, graphene.String)


def test_should_text_convert_string():
    assert_conversion(models.TextField, graphene.String)


def test_should_email_convert_string():
    assert_conversion(models.EmailField, graphene.String)


def test_should_slug_convert_string():
    assert_conversion(models.SlugField, graphene.String)


def test_should_url_convert_string():
    assert_conversion(models.URLField, graphene.String)


def test_should_auto_convert_id():
    assert_conversion(models.AutoField, graphene.ID)


def test_should_positive_integer_convert_int():
    assert_conversion(models.PositiveIntegerField, graphene.Int)


def test_should_positive_small_convert_int():
    assert_conversion(models.PositiveSmallIntegerField, graphene.Int)


def test_should_small_integer_convert_int():
    assert_conversion(models.SmallIntegerField, graphene.Int)


def test_should_big_integer_convert_int():
    assert_conversion(models.BigIntegerField, graphene.Int)


def test_should_integer_convert_int():
    assert_conversion(models.IntegerField, graphene.Int)


def test_should_boolean_convert_boolean():
    field = assert_conversion(models.BooleanField, graphene.Boolean)
    assert field.required is True


def test_should_nullboolean_convert_boolean():
    field = assert_conversion(models.NullBooleanField, graphene.Boolean)
    assert field.required is False


def test_should_float_convert_float():
    assert_conversion(models.FloatField, graphene.Float)


def test_should_manytomany_convert_connectionorlist():
    graphene_type = convert_django_field(Reporter._meta.local_many_to_many[0])
    assert isinstance(graphene_type, ConnectionOrListField)
    assert isinstance(graphene_type.type, DjangoModelField)
    assert graphene_type.type.model == Reporter


def test_should_manytoone_convert_connectionorlist():
    graphene_type = convert_django_field(Reporter.articles.related)
    assert isinstance(graphene_type, ConnectionOrListField)
    assert isinstance(graphene_type.type, DjangoModelField)
    assert graphene_type.type.model == Article


def test_should_onetoone_convert_model():
    field = assert_conversion(models.OneToOneField, DjangoModelField, Article)
    assert field.type.model == Article


def test_should_foreignkey_convert_model():
    field = assert_conversion(models.ForeignKey, DjangoModelField, Article)
    assert field.type.model == Article



from functools import wraps


class CachedPropery(object):
    """
    A property that is only computed once per instance and then replaces itself
    with an ordinary attribute. Deleting the attribute resets the property.
    Source: https://github.com/bottlepy/bottle/commit/fa7733e075da0d790d809aa3d2f53071897e6f76
    """  # noqa

    def __init__(self, func):
        self.__doc__ = getattr(func, '__doc__')
        self.func = func

    def __get__(self, obj, cls):
        if obj is None:
            return self
        value = obj.__dict__[self.func.__name__] = self.func(obj)
        return value

cached_property = CachedPropery


def memoize(fun):
    """A simple memoize decorator for functions supporting positional args."""
    @wraps(fun)
    def wrapper(*args, **kwargs):
        key = (args, frozenset(sorted(kwargs.items())))
        try:
            return cache[key]
        except KeyError:
            ret = cache[key] = fun(*args, **kwargs)
        return ret
    cache = {}
    return wrapper



from graphql.core.type import GraphQLInterfaceType, GraphQLObjectType
from mock import patch

from graphene import Schema
from graphene.contrib.django.types import DjangoInterface, DjangoNode
from graphene.core.fields import Field
from graphene.core.types.scalars import Int
from graphene.relay.fields import GlobalIDField
from tests.utils import assert_equal_lists

from .models import Article, Reporter

schema = Schema()


class Character(DjangoInterface):
    '''Character description'''
    class Meta:
        model = Reporter


@schema.register
class Human(DjangoNode):
    '''Human description'''

    pub_date = Int()

    class Meta:
        model = Article


def test_django_interface():
    assert DjangoNode._meta.is_interface is True


@patch('graphene.contrib.django.tests.models.Article.objects.filter')
def test_django_get_node(objects):
    Human.get_node(1)
    objects.assert_called_with(id=1)


def test_pseudo_interface_registered():
    object_type = schema.T(Character)
    assert Character._meta.is_interface is True
    assert isinstance(object_type, GraphQLInterfaceType)
    assert Character._meta.model == Reporter
    assert_equal_lists(
        object_type.get_fields().keys(),
        ['articles', 'firstName', 'lastName', 'email', 'pets', 'id']
    )


def test_djangonode_idfield():
    idfield = DjangoNode._meta.fields_map['id']
    assert isinstance(idfield, GlobalIDField)


def test_node_idfield():
    idfield = Human._meta.fields_map['id']
    assert isinstance(idfield, GlobalIDField)


def test_node_replacedfield():
    idfield = Human._meta.fields_map['pub_date']
    assert isinstance(idfield, Field)
    assert schema.T(idfield).type == schema.T(Int())


def test_interface_resolve_type():
    resolve_type = Character.resolve_type(schema, Human(object()))
    assert isinstance(resolve_type, GraphQLObjectType)


def test_object_type():
    object_type = schema.T(Human)
    Human._meta.fields_map
    assert Human._meta.is_interface is False
    assert isinstance(object_type, GraphQLObjectType)
    assert_equal_lists(
        object_type.get_fields().keys(),
        ['headline', 'id', 'reporter', 'pubDate']
    )
    assert schema.T(DjangoNode) in object_type.get_interfaces()


def test_node_notinterface():
    assert Human._meta.is_interface is False
    assert DjangoNode in Human._meta.interfaces



from ..resolve_only_args import resolve_only_args


def test_resolve_only_args():
    my_data = {'one': 1, 'two': 2}
    resolver = lambda *args, **kwargs: kwargs
    wrapped = resolve_only_args(resolver)
    assert wrapped(None, my_data, None) == my_data



import warnings

from .types.base import FieldType
from .types.definitions import List, NonNull
from .types.field import Field
from .types.scalars import ID, Boolean, Float, Int, String


class DeprecatedField(FieldType):

    def __init__(self, *args, **kwargs):
        cls = self.__class__
        warnings.warn("Using {} is not longer supported".format(
            cls.__name__), FutureWarning)
        if 'resolve' in kwargs:
            kwargs['resolver'] = kwargs.pop('resolve')
        return super(DeprecatedField, self).__init__(*args, **kwargs)


class StringField(DeprecatedField, String):
    pass


class IntField(DeprecatedField, Int):
    pass


class BooleanField(DeprecatedField, Boolean):
    pass


class IDField(DeprecatedField, ID):
    pass


class FloatField(DeprecatedField, Float):
    pass


class ListField(DeprecatedField, List):
    pass


class NonNullField(DeprecatedField, NonNull):
    pass


__all__ = ['Field', 'StringField', 'IntField', 'BooleanField',
           'IDField', 'FloatField', 'ListField', 'NonNullField']



from functools import total_ordering

import six


class BaseType(object):

    @classmethod
    def internal_type(cls, schema):
        return getattr(cls, 'T', None)


class MountType(BaseType):
    parent = None

    def mount(self, cls):
        self.parent = cls


class LazyType(MountType):

    def __init__(self, type):
        self.type = type

    @property
    def is_self(self):
        return self.type == 'self'

    def internal_type(self, schema):
        type = None
        if callable(self.type):
            type = self.type(self.parent)
        elif isinstance(self.type, six.string_types):
            if self.is_self:
                type = self.parent
            else:
                type = schema.get_type(self.type)
        assert type, 'Type in %s %r cannot be none' % (self.type, self.parent)
        return schema.T(type)


@total_ordering
class OrderedType(MountType):
    creation_counter = 0

    def __init__(self, _creation_counter=None):
        self.creation_counter = _creation_counter or self.gen_counter()

    @staticmethod
    def gen_counter():
        counter = OrderedType.creation_counter
        OrderedType.creation_counter += 1
        return counter

    def __eq__(self, other):
        # Needed for @total_ordering
        if isinstance(self, type(other)):
            return self.creation_counter == other.creation_counter
        return NotImplemented

    def __lt__(self, other):
        # This is needed because bisect does not take a comparison function.
        if isinstance(other, OrderedType):
            return self.creation_counter < other.creation_counter
        return NotImplemented

    def __gt__(self, other):
        # This is needed because bisect does not take a comparison function.
        if isinstance(other, OrderedType):
            return self.creation_counter > other.creation_counter
        return NotImplemented

    def __hash__(self):
        return hash((self.creation_counter))


class MirroredType(OrderedType):

    def __init__(self, *args, **kwargs):
        _creation_counter = kwargs.pop('_creation_counter', None)
        super(MirroredType, self).__init__(_creation_counter=_creation_counter)
        self.args = args
        self.kwargs = kwargs

    @property
    def List(self):  # noqa
        from .definitions import List
        return List(self, *self.args, **self.kwargs)

    @property
    def NonNull(self):  # noqa
        from .definitions import NonNull
        return NonNull(self, *self.args, **self.kwargs)


class ArgumentType(MirroredType):

    def as_argument(self):
        from .argument import Argument
        return Argument(
            self, _creation_counter=self.creation_counter, *self.args, **self.kwargs)


class FieldType(MirroredType):

    def contribute_to_class(self, cls, name):
        from ..types import BaseObjectType, InputObjectType
        if issubclass(cls, InputObjectType):
            inputfield = self.as_inputfield()
            return inputfield.contribute_to_class(cls, name)
        elif issubclass(cls, BaseObjectType):
            field = self.as_field()
            return field.contribute_to_class(cls, name)

    def as_field(self):
        from .field import Field
        return Field(self, _creation_counter=self.creation_counter,
                     *self.args, **self.kwargs)

    def as_inputfield(self):
        from .field import InputField
        return InputField(
            self, _creation_counter=self.creation_counter, *self.args, **self.kwargs)


class MountedType(FieldType, ArgumentType):
    pass



from py.test import raises

import graphene
from graphene import relay
from graphene.contrib.django import DjangoNode, DjangoObjectType

from .models import Article, Reporter


def test_should_query_only_fields():
    with raises(Exception):
        class ReporterType(DjangoObjectType):

            class Meta:
                model = Reporter
                only_fields = ('articles', )

        schema = graphene.Schema(query=ReporterType)
        query = '''
            query ReporterQuery {
              articles
            }
        '''
        result = schema.execute(query)
        assert not result.errors


def test_should_query_well():
    class ReporterType(DjangoObjectType):

        class Meta:
            model = Reporter

    class Query(graphene.ObjectType):
        reporter = graphene.Field(ReporterType)

        def resolve_reporter(self, *args, **kwargs):
            return ReporterType(Reporter(first_name='ABA', last_name='X'))

    query = '''
        query ReporterQuery {
          reporter {
            firstName,
            lastName,
            email
          }
        }
    '''
    expected = {
        'reporter': {
            'firstName': 'ABA',
            'lastName': 'X',
            'email': ''
        }
    }
    schema = graphene.Schema(query=Query)
    result = schema.execute(query)
    assert not result.errors
    assert result.data == expected


def test_should_node():
    class ReporterNode(DjangoNode):

        class Meta:
            model = Reporter

        @classmethod
        def get_node(cls, id):
            return ReporterNode(Reporter(id=2, first_name='Cookie Monster'))

        def resolve_articles(self, *args, **kwargs):
            return [ArticleNode(Article(headline='Hi!'))]

    class ArticleNode(DjangoNode):

        class Meta:
            model = Article

        @classmethod
        def get_node(cls, id):
            return ArticleNode(Article(id=1, headline='Article node'))

    class Query(graphene.ObjectType):
        node = relay.NodeField()
        reporter = graphene.Field(ReporterNode)
        article = graphene.Field(ArticleNode)

        def resolve_reporter(self, *args, **kwargs):
            return ReporterNode(
                Reporter(id=1, first_name='ABA', last_name='X'))

    query = '''
        query ReporterQuery {
          reporter {
            id,
            firstName,
            articles {
              edges {
                node {
                  headline
                }
              }
            }
            lastName,
            email
          }
          myArticle: node(id:"QXJ0aWNsZU5vZGU6MQ==") {
            id
            ... on ReporterNode {
                firstName
            }
            ... on ArticleNode {
                headline
            }
          }
        }
    '''
    expected = {
        'reporter': {
            'id': 'UmVwb3J0ZXJOb2RlOjE=',
            'firstName': 'ABA',
            'lastName': 'X',
            'email': '',
            'articles': {
                'edges': [{
                  'node': {
                      'headline': 'Hi!'
                  }
                }]
            },
        },
        'myArticle': {
            'id': 'QXJ0aWNsZU5vZGU6MQ==',
            'headline': 'Article node'
        }
    }
    schema = graphene.Schema(query=Query)
    result = schema.execute(query)
    assert not result.errors
    assert result.data == expected
